@@ -58,6 +58,8 @@ else:
         "bl602": "bl602",
         "bl702": "bl702",
         "bl808": "bl808",
+        "bl616": "bl616",
+        "wb03" : "wb03",
     }
     chip_xtal = 'bl60x_xtal'
     chip_brd = 'bl60x_brd'
@@ -65,6 +67,7 @@ else:
    
     
 boot2_dir = 'boot2_iap_v5.0'
+ENABLE_HAIER = False
 
 
 def parse_rfpa(bin):
@@ -132,6 +135,40 @@ def generate_romfs_img(romfs_dir, dst_img_name):
     return subprocess.call([exe, '-d', dir, '-f', dst])
 
 
+def check_pt_table(pt_parcel, file):
+    if len(file) > 0:
+        i = 0
+        try:
+            while i < len(file):
+                file_name = file[i]
+                file_size = os.path.getsize(os.path.join(app_path, file_name))
+                if "whole_img_boot2.bin" in file_name:
+                    if file_size > pt_parcel['pt_addr0']:
+                        bflb_utils.printf("Error: boot2 bin size is overflow with partition table")
+                        return False
+                if "whole_img_cpu0.bin" in file_name:
+                    if file_size > pt_parcel['fw_cpu0_len']:
+                        bflb_utils.printf("Error: cpu0 fw bin size is overflow with partition table")
+                        return False
+                if "whole_img.bin" in file_name:
+                    if file_size > pt_parcel['fw_len']:
+                        bflb_utils.printf("Error: fw bin size is overflow with partition table")
+                        return False
+                if "whole_img_mfg.bin" in file_name:
+                    if file_size > pt_parcel['mfg_len']:
+                        bflb_utils.printf("Error: mfg bin size is overflow with partition table")
+                        return False
+                if "media.bin" in file_name:
+                    if file_size > pt_parcel['media_len']:
+                        bflb_utils.printf("Error: media bin size is overflow with partition table")
+                        return False
+                i += 1
+        except Exception as e:
+            bflb_utils.printf(e)
+            return False
+    return True
+
+
 def exe_genitor(list_args):
     p = subprocess.Popen(list_args, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
     while subprocess.Popen.poll(p) is None:
@@ -243,8 +280,8 @@ class BflbIotTool(object):
     
         cfg.write(config)
         eb_create.efuse_boothd_create_process(chipname, chiptype, config)
-    
-    
+
+
     def bl60x_mfg_boot_head_gen(self,
                                 mfg_addr,
                                 xtal,
@@ -412,6 +449,20 @@ class BflbIotTool(object):
         return header
 
 
+    def bl60x_mfg_ota_header_haier(self, fw_ota_bin, chipname="bl60x", version="", dir_ota=None):        
+        # fw_ota_bin_ver_haier = '525f362e302e30302e652d55444953434f564552595f55575400000000000000'
+        fw_ota_bin_ver_haier = 'M_1.0.00-e-UDISCOVERY_UWT'
+        fw_ota_bin_sn_haier = '00'*16
+        fw_ota_bin_md5 = hashlib.md5(fw_ota_bin).hexdigest()      
+        fw_ota_bin_haier = bytes.fromhex(fw_ota_bin_ver_haier.encode('utf-8').hex() + '00'*7 + fw_ota_bin_sn_haier + fw_ota_bin_md5.encode('utf-8').hex() + version.encode('utf-8').hex() + '00'*(48-len(version)))        
+        if dir_ota:
+            fw_ota_path_haier = os.path.join(dir_ota, "FW_OTA_Haier.bin.ota")
+        else:
+            fw_ota_path_haier = os.path.join(chip_path, chipname, "ota", "FW_OTA_Haier.bin.ota")
+        with open(fw_ota_path_haier, mode="wb") as f:
+            f.write(fw_ota_bin_haier + fw_ota_bin)
+            
+
     def bl60x_mfg_ota_xz_gen(self, ota_path, chipname="bl60x", chiptype="bl60x", cpu_type=None):
         bl60x_xz_filters = [
             {
@@ -455,10 +506,10 @@ class BflbIotTool(object):
         with open(fw_ota_path.replace(".ota", ".hash"), mode="wb") as f:
             f.write(fw_ota_bin_xz + img_create_sha256_data(fw_ota_bin_xz))
         with open(fw_ota_path1, mode="wb") as f:
-            f.write(fw_ota_bin + img_create_sha256_data(fw_ota_bin_xz))
+            f.write(fw_ota_bin + img_create_sha256_data(fw_ota_bin))
     
     
-    def bl60x_mfg_ota_bin_gen(self, chipname="bl60x", chiptype="bl60x", cpu_type=None, version=None, dir_ota=None):
+    def bl60x_mfg_ota_bin_gen(self, chipname="bl60x", chiptype="bl60x", cpu_type=None, version="", dir_ota=None):
         fw_header_len = 4096
         fw_ota_bin = bytearray()
         if dir_ota: 
@@ -492,6 +543,9 @@ class BflbIotTool(object):
             file_bytes = f.read()
             for b in file_bytes:
                 fw_ota_bin.append(b)
+        # add header for Haier 
+        if ENABLE_HAIER:
+            self.bl60x_mfg_ota_header_haier(fw_ota_bin, chipname, version, dir_ota)
         fw_ota_bin_header = self.bl60x_mfg_ota_header(chipname, fw_ota_bin, use_xz=0)
         if cpu_type is None:
             fw_ota_path = os.path.join(ota_path, "FW_OTA.bin")
@@ -568,6 +622,18 @@ class BflbIotTool(object):
         cfg.set('LOAD_CFG', 'interface', 'jlink')
         cfg.set('LOAD_CFG', 'speed_jlink', rate)
         cfg.write(f, 'w')
+        
+        
+    def bl60x_mfg_flasher_cklink_cfg(self, rate="1000", cfg_ini=None):
+        cfg = BFConfigParser()
+        if cfg_ini in [None, '']:
+            f = self.eflash_loader_cfg
+        else:
+            f = cfg_ini
+        cfg.read(f)
+        cfg.set('LOAD_CFG', 'interface', 'cklink')
+        cfg.set('LOAD_CFG', 'speed_jlink', rate)
+        cfg.write(f, 'w')
     
     
     def bl60x_mfg_flasher_openocd_cfg(self, rate="8000", cfg_ini=None):
@@ -738,6 +804,9 @@ class BflbIotTool(object):
                 d_addrs.append(hex(pt_parcel['mfg_addr'] + 0x1000)[2:])
     
         if len(d_files) > 0 and len(d_addrs) > 0:
+            if check_pt_table(pt_parcel, d_files) != True:
+                bflb_utils.set_error_code("0082")
+                return bflb_utils.errorcode_msg()
             cfg = BFConfigParser()
             cfg.read(self.eflash_loader_cfg)
             self.bl_write_flash_img(d_addrs, d_files, flash_opt)
@@ -1022,12 +1091,23 @@ class BflbIotTool(object):
             if config['param']['interface_type'] == 'Uart':
                 uart = config['param']['comport_uart']
                 uart_brd = config['param']['speed_uart']
+                if not uart_brd.isdigit():
+                    error = '{"ErrorCode":"FFFF","ErrorMsg":"BAUDRATE MUST BE DIGIT"}'
+                    bflb_utils.printf(error)
+                    return error                    
                 bflb_utils.printf("========= Interface is Uart =========")
                 self.bl60x_mfg_flasher_cfg(uart, uart_brd)
             elif config['param']['interface_type'] == 'JLink':
                 jlink_brd = config['param']['speed_jlink']
+                if not jlink_brd.isdigit():
+                    error = '{"ErrorCode":"FFFF","ErrorMsg":"BAUDRATE MUST BE DIGIT"}'
+                    bflb_utils.printf(error)
+                    return error     
                 bflb_utils.printf("========= Interface is JLink =========")
                 self.bl60x_mfg_flasher_jlink_cfg(rate=jlink_brd)
+            elif config['param']['interface_type'] == 'CKLink':
+                bflb_utils.printf("========= Interface is CKLink =========")
+                self.bl60x_mfg_flasher_cklink_cfg()
             else:
                 openocd_brd = config['param']['speed_jlink']
                 bflb_utils.printf("========= Interface is Openocd =========")
@@ -1059,6 +1139,10 @@ class BflbIotTool(object):
                 cfg.set('FLASH_CFG', 'address', addrs_str)
                 cfg.write(self.eflash_loader_cfg, 'w')
                 self.eflash_loader_t = eflash_loader.BflbEflashLoader(chipname, chiptype)
+                if not config["param"]["comport_uart"] and config['param']['interface_type'] == 'Uart':
+                    error = '{"ErrorCode":"FFFF","ErrorMsg":"BFLB INTERFACE HAS NO COM PORT"}'
+                    bflb_utils.printf(error)
+                    return error
                 if config["check_box"]['ckb_erase_all'] == "True":
                     self.bl60x_mfg_flasher_erase_all(True)
                 else:
@@ -1081,6 +1165,10 @@ class BflbIotTool(object):
                     # burn into flash
                     self.eflash_loader_t = eflash_loader.BflbEflashLoader(chipname, chiptype)
                     self.eflash_loader_t.set_config_file(self.bh_cfg_file, "")
+                    if not config["param"]["comport_uart"] and config['param']['interface_type'] == 'Uart':
+                        error = '{"ErrorCode":"FFFF","ErrorMsg":"BFLB INTERFACE HAS NO COM PORT"}'
+                        bflb_utils.printf(error)
+                        return error
                     if config["check_box"]['ckb_erase_all'] == "True":
                         self.bl60x_mfg_flasher_erase_all(True)
                     else:
@@ -1180,16 +1268,24 @@ def iot_download_cmd(args):
     config["check_box"]["download_single"] = args.single
     config["input_path"]["img_bin_input"] = args.firmware
     config["param"]["addr"] = "0x" + args.addr
-    config["param"]["version"] = ""
-    config["param"]["ota"] = ""
+#     config["param"]["version"] = ""
+#     config["param"]["ota"] = ""
+    
+    if not args.version:
+        config["param"]["version"] = ""    
+    else:
+        config["param"]["version"] = args.version
+    if not args.ota:
+        config["param"]["ota"] = os.path.join(chip_path, chipname, "ota")
+    else:
+        config["param"]["ota"] = args.ota
 
-    if not os.path.exists(args.pt):
+    if not args.pt or not os.path.exists(args.pt):
         config["input_path"]["pt_bin_input"] = os.path.join(chip_path, chipname, "partition/partition_cfg_2M.toml") 
     else:
         config["input_path"]["pt_bin_input"] = args.pt
     bflb_utils.printf("Partition table is " + config["input_path"]["pt_bin_input"])      
 
-    
     if chiptype == "bl702":
         config["check_box"]["boot2_download"] = False
         config["input_path"]["boot2_bin_input"] = ""
@@ -1197,7 +1293,7 @@ def iot_download_cmd(args):
             config["param"][chip_xtal] = "32M"
         else:   
             config["param"][chip_xtal] = args.xtal 
-        if not os.path.exists(args.dts): 
+        if not args.dts or not os.path.exists(args.dts): 
             config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", bl_factory_params_file_prefix+"IoTKitA_32M.dts")
         else:   
             config["input_path"]["dts_input"] = args.dts
@@ -1208,7 +1304,7 @@ def iot_download_cmd(args):
             config["param"][chip_xtal] = "40M"
         else:   
             config["param"][chip_xtal] = args.xtal 
-        if not os.path.exists(args.dts): 
+        if not args.dts or not os.path.exists(args.dts): 
             config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", bl_factory_params_file_prefix+"IoTKitA_40M.dts")
         else:   
             config["input_path"]["dts_input"] = args.dts
@@ -1219,7 +1315,7 @@ def iot_download_cmd(args):
             config["param"][chip_xtal] = "38.4M"
         else:   
             config["param"][chip_xtal] = args.xtal 
-        if not os.pasth.exists(args.dts): 
+        if not args.dts or not os.path.exists(args.dts):  
             config["input_path"]["dts_input"] = os.path.join(chip_path, chipname, "device_tree", bl_factory_params_file_prefix+"avb_38.4M.dts")
         else:   
             config["input_path"]["dts_input"] = args.dts
@@ -1272,8 +1368,8 @@ def run():
     parser.add_argument("--single", dest="single", action="store_true", help="single download")
     parser.add_argument("--addr", dest="addr", default="0", help="address to write")
     parser.add_argument('--config', dest="config", help='config file')
-    #parser.add_argument('--ota', dest="ota", help='ota dir')
-    #parser.add_argument('--version', dest="version", help='version')
+    parser.add_argument('--ota', dest="ota", help='ota dir')
+    parser.add_argument('--version', dest="version", help='version')
     args = parser.parse_args()
     bflb_utils.printf("==================================================")
     bflb_utils.printf("Chip name is %s" % args.chipname)  
